@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2008 NTT DATA CORPORATION.
+ * Copyright (c) since 2006 NTT DATA CORPORATION.
  * All rights reserved.
  */
 
@@ -342,10 +342,12 @@ update_index_with_datum(index_info *ii, const void *key, unsigned int section,
       bits8 *bitmap = ARR_NULLBITMAP(atype);
 #endif
       int bitmask = 1;
+      int null_flag = 0;
       for (i = 0; i < *ARR_DIMS(atype); i++) {
         if (!bitmap || (*bitmap & bitmask)) {
           char *c = datum2cstr(fetch_att(v_ptr, false, -1));
           update_index_with_char(ii, key, i, c, NULL);
+          null_flag = 1;
 #if defined(POSTGRES81) || defined(POSTGRES82)
           v_ptr = att_addlength(v_ptr, -1, PointerGetDatum(v_ptr));
           v_ptr = (char *) att_align(v_ptr, 'i');
@@ -364,6 +366,12 @@ update_index_with_datum(index_info *ii, const void *key, unsigned int section,
           }
         }
       }
+      // If no records is inserted into fulltext-index, 
+      // records of index differ from records of table.
+      // So, "" is inserted into fulltext-index.
+      if (null_flag == 0) {
+        update_index_with_char(ii, key, i, "", NULL);
+      }
     }
     if (newv) {
       ArrayType* atype = DatumGetArrayTypeP(newv);
@@ -375,6 +383,7 @@ update_index_with_datum(index_info *ii, const void *key, unsigned int section,
       bits8 *bitmap = ARR_NULLBITMAP(atype);
 #endif
       int bitmask = 1;
+      int null_flag = 0;
       for (i = 0; i < *ARR_DIMS(atype); i++) {
         if (!bitmap || (*bitmap & bitmask)) {
           char *c = datum2cstr(fetch_att(v_ptr, false, -1));
@@ -397,6 +406,9 @@ update_index_with_datum(index_info *ii, const void *key, unsigned int section,
           }
         }
       }
+      if (null_flag == 0) {
+        update_index_with_char(ii, key, i, NULL, "");
+      }
     }
   }
 }
