@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2008 NTT DATA CORPORATION.
+ * Copyright (c) since 2006 NTT DATA CORPORATION.
  * All rights reserved.
  *
  * derived from original pg_senna.c developed by Daisuke Maki
@@ -152,7 +152,7 @@ typedef struct {
 
 inline static void *scan_stat_open(sen_records *records, index_info *ii);
 inline static void scan_stat_close(void *opaque);
-void __attribute__ ((constructor)) pgs2_init(void);
+void _PG_init(void);
 Datum pgs2build0(Relation heap, Relation index, IndexInfo *indexInfo,
                  int flags, Oid oid_type);
 Datum pgs2insert0(Relation r, Datum *values, bool *isnull,
@@ -183,7 +183,10 @@ static sen_logger_info pgs2_sen_logger_info = {sen_log_warning,
                                                pgs2_logger_func};
 
 
-void __attribute__ ((constructor)) pgs2_init(void) {
+/*
+ * _PG_init is called when pgsenna2.lib is loaded, without PostgreSQL8.1
+ */
+void _PG_init (void) {
   sen_rc rc;
   char option[16];
   snprintf(option, 16, "%d", MAX_N_SORT_RESULT);
@@ -233,6 +236,10 @@ void __attribute__ ((constructor)) pgs2_init(void) {
 };
 
 
+/*
+ * sen_check_init is called when CREATE INDEX and SELECT(index-scan) is
+ * executed at first of session.
+ */
 void
 sen_check_init(void)
 {
@@ -242,9 +249,10 @@ sen_check_init(void)
   const char *option;
 
   if (initialized) { return ; }
-#ifndef __GNUC__
-  pgs2_init();
-#endif /* __GNUC__ */
+
+#ifdef POSTGRES81
+  _PG_init ();
+#endif /* POSTGRES81 */
 
   option = getludiaoption("ludia.max_n_index_cache");
   if (option) {
@@ -275,6 +283,10 @@ sen_check_init(void)
 }
 
 
+/*
+ * index_list_update is called by index_info_open.
+ * It is LRU algorithm function for open-close index.
+ */
 inline static void
 index_list_update(index_info *ii) {
   if (last_used_cache == ii) {
@@ -298,6 +310,11 @@ index_list_update(index_info *ii) {
 /* todo: SizeOfIptrData may be more suitable.  */
 #define KEY_SIZE (sizeof(ItemPointerData))
 
+
+/*
+ * index_info_open is called when index-operation.
+ * Index-operation is CREATE INDEX, SELECT(index-scan), UPDATE, INSERT.
+ */
 index_info *
 index_info_open(Relation r, int createp, int flags)
 {
@@ -307,6 +324,7 @@ index_info_open(Relation r, int createp, int flags)
   char path[MAXPGPATH];
   Oid namespace = RelationGetNamespace(r);
   Oid relid = RelationGetRelid(r);
+  RelFileNode rfn = r->rd_node;
   char *name = RelationGetRelationName(r);
 
   sen_check_init();
@@ -316,6 +334,7 @@ index_info_open(Relation r, int createp, int flags)
     if (ii->index &&
         namespace == ii->namespace &&
         relid == ii->relid &&
+        RelFileNodeEquals(rfn, ii->rfn) &&
         !strcmp(ii->name, name)) {
       if (createp) { /* for REINDEX (and CLUSTER) */
         break;
@@ -351,6 +370,7 @@ index_info_open(Relation r, int createp, int flags)
   }
   cand->namespace = namespace;
   cand->relid = relid;
+  cand->rfn = rfn;
   strcpy(cand->name, name);
   {
     char *pathname;
@@ -431,6 +451,10 @@ index_info_close(index_info *ii)
 }
 
 
+/*
+ * scan_stat_open is called when SELECT is executed.
+ * It connects result-set of SELECT(index-scan) to index-info.
+ */
 inline static void *
 scan_stat_open(sen_records *records, index_info *ii)
 {
@@ -445,6 +469,9 @@ scan_stat_open(sen_records *records, index_info *ii)
 }
 
 
+/*
+ * scan_stat_close frees result-set of SELECT.
+ */
 inline static void
 scan_stat_close(void *opaque)
 {
@@ -459,6 +486,9 @@ scan_stat_close(void *opaque)
 }
 
 
+/*
+ * scan_stat_close frees result-set on memory.
+ */
 inline static void
 scan_stat_close_all()
 {
@@ -486,6 +516,11 @@ init_support_funcs(support_funcs *funcs, Relation index)
                  CurrentMemoryContext);
 }
 
+
+/*
+ * do_insert is called when UPDATE or INSERT or CREATE INDEX is executed.
+ * It insert PostgreSQL-row into senna-index.
+ */
 inline static void
 do_insert(Relation r, ItemPointer tid, index_info *ii,
           Datum *values, bool *isnull, support_funcs *funcs, Oid oid_type)
@@ -539,6 +574,9 @@ do_insert(Relation r, ItemPointer tid, index_info *ii,
 }
 
 
+/*
+ * buildCallback registers do_insert on callback-func(INSERT or UPDATE)
+ */
 static void
 buildCallback(Relation index,
               HeapTuple htup,
@@ -554,6 +592,10 @@ buildCallback(Relation index,
 }
 
 
+/*
+ * pgs2build0 is called by pgs2build, _pgs2build, pgs2buildb, _pgs2buildb,
+ * pgs2buildu, _pgs2buildu.
+ */
 Datum
 pgs2build0(Relation heap, Relation index, IndexInfo *indexInfo, int flags, Oid oid_type)
 {
@@ -583,6 +625,9 @@ pgs2build0(Relation heap, Relation index, IndexInfo *indexInfo, int flags, Oid o
 }
 
 
+/*
+ * pgs2build is called when CREATE INDEX using fulltext().
+ */
 Datum
 pgs2build(PG_FUNCTION_ARGS)
 {
@@ -593,6 +638,9 @@ pgs2build(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * _pgs2build is called when CREATE INDEX using fulltexta().
+ */
 Datum
 _pgs2build(PG_FUNCTION_ARGS)
 {
@@ -603,6 +651,9 @@ _pgs2build(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2buildb is called when CREATE INDEX using fulltextb().
+ */
 Datum
 pgs2buildb(PG_FUNCTION_ARGS)
 {
@@ -614,6 +665,9 @@ pgs2buildb(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * _pgs2buildb is called when CREATE INDEX using fulltextab().
+ */
 Datum
 _pgs2buildb(PG_FUNCTION_ARGS)
 {
@@ -625,6 +679,9 @@ _pgs2buildb(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2buildu is called when CREATE INDEX using fulltextu().
+ */
 Datum
 pgs2buildu(PG_FUNCTION_ARGS)
 {
@@ -645,6 +702,9 @@ pgs2buildu(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2build is called when CREATE INDEX using _fulltextu().
+ */
 Datum
 _pgs2buildu(PG_FUNCTION_ARGS)
 {
@@ -665,6 +725,9 @@ _pgs2buildu(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2insert0 is called by pgs2insert or _pgs2insert.
+ */
 Datum
 pgs2insert0(Relation r, Datum *values, bool *isnull,
             ItemPointer ht_ctid, Oid oid_type)
@@ -709,6 +772,9 @@ _pgs2insert(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2bulkdelete0 is called by pgs2bulkdelete or _pgs2bulkdelete.
+ */
 Datum
 pgs2bulkdelete0(Relation r, IndexBulkDeleteCallback callback,
                 void *callback_state, IndexBulkDeleteResult *stats, Oid oid_type)
@@ -774,6 +840,10 @@ pgs2bulkdelete0(Relation r, IndexBulkDeleteCallback callback,
 }
 
 
+/*
+ * pgs2builkdelete is called when VACUUM(fulltext,fulltextb,fulltextu)
+ * is executed.
+ */
 Datum
 pgs2bulkdelete(PG_FUNCTION_ARGS)
 {
@@ -793,6 +863,10 @@ pgs2bulkdelete(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * _pgs2builkdelete is called when VACUUM(fulltexta,fulltextab,fulltextau)
+ * is executed.
+ */
 Datum
 _pgs2bulkdelete(PG_FUNCTION_ARGS)
 {
@@ -836,6 +910,9 @@ pgs2vacuumcleanup(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2beginscan is called when SELECT(index-scan) is executed.
+ */
 Datum
 pgs2beginscan(PG_FUNCTION_ARGS)
 {
@@ -851,6 +928,10 @@ pgs2beginscan(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2gettuple is called when SELECT(index-scan) is executed.
+ * pgs2gettuple gets results of senna-index-scan, and returns the results to PG.
+ */
 Datum
 pgs2gettuple(PG_FUNCTION_ARGS)
 {
@@ -897,6 +978,11 @@ pgs2gettuple(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2getmulti is called when SELECT(index-scan) is executed.
+ * pgs2getmulti is similar to pgs2gettuple.
+ * pgs2gettuple returns single tuple, pgs2getmulti returns multi tuples.
+ */
 Datum
 pgs2getmulti(PG_FUNCTION_ARGS)
 {
@@ -939,6 +1025,11 @@ pgs2getmulti(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2rescan is called when SELECT(index-scan) is executed.
+ * pgs2beginscan is called before pgs2rescan.
+ * pgs2endscan is called after pgs2rescan.
+ */
 Datum
 pgs2rescan(PG_FUNCTION_ARGS)
 {
@@ -1029,6 +1120,9 @@ pgs2rescan(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2endscan is called when SELECT(index-scan) is executed.
+ */
 Datum
 pgs2endscan(PG_FUNCTION_ARGS)
 {
@@ -1044,6 +1138,9 @@ pgs2endscan(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2markpos isn't used...
+ */
 Datum
 pgs2markpos(PG_FUNCTION_ARGS)
 {
@@ -1061,6 +1158,9 @@ pgs2markpos(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2restrpos isn't used...
+ */
 Datum
 pgs2restrpos(PG_FUNCTION_ARGS)
 {
@@ -1079,6 +1179,12 @@ pgs2restrpos(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2costestimate is called when "WHERE column %% 'word'" is executed.
+ * It returns the cost of index-scan.
+ * PostgreSQL compares the cost of index-scan with the cost of seq-scan.
+ * Lower-cost plan is selected.
+ */
 Datum
 pgs2costestimate(PG_FUNCTION_ARGS)
 {
@@ -1200,6 +1306,9 @@ pgs2costestimate(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2contain0 is called by pgs2contain or _pgs2contain.
+ */
 Datum
 pgs2contain0(Datum vt, Datum vq, Oid oid_type)
 {
@@ -1308,6 +1417,10 @@ pgs2contain0(Datum vt, Datum vq, Oid oid_type)
 }
 
 
+/*
+ * pgs2contain is called when SELECT(seq-scan of fulltext,fulltextb,fulltextu)
+ * is executed.
+ */
 Datum
 pgs2contain(PG_FUNCTION_ARGS)
 {
@@ -1318,6 +1431,10 @@ pgs2contain(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * _pgs2contain is called when SELECT(seq-scan of fulltexta,fulltextab,
+ * fulltextau) is executed.
+ */
 Datum
 _pgs2contain(PG_FUNCTION_ARGS)
 {
@@ -1335,6 +1452,9 @@ pgs2nop(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2getscore is called when pgs2getscore(index-name, ctid) is executed.
+ */
 Datum
 pgs2getscore(PG_FUNCTION_ARGS)
 {
@@ -1361,6 +1481,9 @@ pgs2getscore(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2getscore_simple is called when pgs2getscore(ctid) is executed.
+ */
 Datum
 pgs2getscore_simple(PG_FUNCTION_ARGS)
 {
@@ -1383,6 +1506,9 @@ pgs2getnhits(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * pgs2options isn't used...
+ */
 #if defined(POSTGRES82) || defined(POSTGRES83)
 Datum
 pgs2options(PG_FUNCTION_ARGS)
@@ -1404,6 +1530,10 @@ pgs2options(PG_FUNCTION_ARGS)
 #endif
 
 
+/*
+ * pgs2indexcache shows the indexes on the memory of the process.
+ * It doesn't show the indexes on the memory of another process.
+ */
 Datum
 pgs2indexcache(PG_FUNCTION_ARGS)
 {
